// ProgressDlg.cpp : implementation file
//

#include "stdafx.h"
#include "rootfly.h"
#include "ProgressDlg.h"
#include "DataStructures.h"

#include <sstream>
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// This cursor should be defined in <winuser.h>, but it will not if WINVER is not greater than 0500.
// But if you manually set WINVER=0500, you will get lots of errors, unless you change the order in
// which your include directories are used.  But if you do this, then *every* developer will have to
// do the same, just to compile the code (a big headache).  See http://forum.ultravnc.info/viewtopic.php?t=6929.
// So I propose this simpler solution.
#ifndef IDC_HAND 
#define IDC_HAND            MAKEINTRESOURCE(32649)
#endif


/////////////////////////////////////////////////////////////////////////////
// ProgressDlg dialog


ProgressDlg::ProgressDlg(const Experiment &experiment, const CString &tube, CWnd* pParent /*=NULL*/)
: CDialog(ProgressDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(ProgressDlg)
	//}}AFX_DATA_INIT

  m_tube_list = experiment.GetAllTubes();
  m_max_session = experiment.GetMaxSessions();
  m_max_window = experiment.GetMaxWindows();
  m_selected_tube = m_selected_session = m_selected_window = -1;
  m_displayed_tube = tube;
  m_starting_session = 1;
  m_starting_window = 1;
  m_jump_size = 5;
  m_columns = 27;
  m_rows = 15;

  COLOR_NOT_EXIST = RGB(200,200,200);
  COLOR_NOT_ANNOTATED = RGB(255,0,0);
  COLOR_DONE = RGB(0,255,0);

  m_window_size = CSize(20, 15);
  m_tool_tip.Create(this);
}


void ProgressDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(ProgressDlg)
	DDX_Control(pDX, IDC_DOWN, m_down_button);
	DDX_Control(pDX, IDC_LEFT, m_left_button);
	DDX_Control(pDX, IDC_RIGHT, m_right_button);
	DDX_Control(pDX, IDC_UP, m_up_button);
	DDX_Control(pDX, IDC_PROGRESS_SCROLL_TUBES, m_sTubeSelect);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(ProgressDlg, CDialog)
	//{{AFX_MSG_MAP(ProgressDlg)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_HSCROLL()
	ON_WM_PAINT()
	ON_BN_CLICKED(IDC_UP, OnClickUp)
	ON_BN_CLICKED(IDC_RIGHT, OnClickRight)
	ON_BN_CLICKED(IDC_LEFT, OnClickLeft)
	ON_BN_CLICKED(IDC_DOWN, OnClickDown)
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// ProgressDlg message handlers

BOOL ProgressDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here

  InitSelection();  
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


void ProgressDlg::InitSelection()
{
  int displayed_tube;

  if (m_tube_list->size() <= 14)
    m_sTubeSelect.EnableScrollBar(ESB_DISABLE_BOTH);
  else
    m_sTubeSelect.SetScrollRange(0,m_tube_list->size()-14);

  for(int i = 0; i < m_tube_list->size(); i++)
  {
	  if (m_tube_list->at(i).GetTag() == m_displayed_tube)
		  displayed_tube = i;
  }

  if (displayed_tube >= 14)
    m_sTubeSelect.SetScrollPos(displayed_tube-14+1);
}

void ProgressDlg::SelectImage(const int &tube, const int &session, const int &window)
{
  m_selected_tube = tube;
  m_selected_session = session;
  m_selected_window = window;

  OnOK();
}

void ProgressDlg::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CRect client;
	GetClientRect(&client);
	
	int curpos = m_sTubeSelect.GetScrollPos();
	int i, j, k, p, tube_index;   //loop counters
	int col_length = 22;
	int row_length = 17;
	int columns = 27;
	int rows = 15;
  int check = 0;

	//Determine index of current tube in tube vector
	for(i = 0; i < m_tube_list->size(); i++)
	{
		if (m_tube_list->at(i).GetTag() == m_displayed_tube)
			tube_index = i;
	}
	

	for(i = m_starting_window, k = 1; i<columns + m_starting_window - 1; i++, k++)
	{
		for(j = m_starting_session, p = 1; j<rows + m_starting_session - 1; j++, p++)
		{
			//First check if image exists
			if((m_tube_list->at(tube_index).GetImageExists(j, i))){
				CRect tube_box = new CRect(CPoint((k*col_length), (p*row_length)) + CSize(25,40), m_window_size);
				if(tube_box.PtInRect(point))
				{
					SelectImage(tube_index, i, j);
          check = 1;
				}
			}
		}
	}
	
	
	// Check for click in tube selector
	for (i=0+curpos; (i<14+curpos); i++)
	{
		CRect tube_box = new CRect(CPoint((i-curpos)*40+60,client.Height()-150), CSize(30, 20));
		if (tube_box.PtInRect(point))
		{
			if (i < m_tube_list->size())
				m_displayed_tube = m_tube_list->at(i).GetTag();
        check = 1;
		}
	}

  if (check == 1)
	  Invalidate();

	CDialog::OnLButtonDown(nFlags, point);
}

void ProgressDlg::OnPaint() 
{

	ReDraw();	
  CDialog::OnPaint();
}

void ProgressDlg::OnFinalRelease() 
{
	// TODO: Add your specialized code here and/or call the base class
	
	CDialog::OnFinalRelease();
}

BOOL ProgressDlg::PreTranslateMessage(MSG* pMsg) 
{
	m_tool_tip.RelayEvent(pMsg);
	
	return CDialog::PreTranslateMessage(pMsg);
}

void ProgressDlg::OnMouseMove(UINT nFlags, CPoint point) 
{
  CRect client;
	GetClientRect(&client);
	
	int curpos = m_sTubeSelect.GetScrollPos();
	int i, j, k, p, tube_index;   //loop counters
	int col_length = 22;
	int row_length = 17;
	int columns = 27;
	int rows = 15;
  int check = 0;
  CString text;

	//Determine index of current tube in tube vector
	for(i = 0; i < m_tube_list->size(); i++)
	{
		if (m_tube_list->at(i).GetTag() == m_displayed_tube)
			tube_index = i;
	}
	

	for(i = m_starting_window, k = 1; i<columns + m_starting_window - 1; i++, k++)
	{
		for(j = m_starting_session, p = 1; j<rows + m_starting_session - 1; j++, p++)
		{
			//First check if image exists
			if((m_tube_list->at(tube_index).GetImageExists(j, i))){
				CRect tube_box = new CRect(CPoint((k*col_length), (p*row_length)) + CSize(25,40), m_window_size);
				if(tube_box.PtInRect(point))
				{
		      text.Format("Window %d, Session %d",j,i);
          check = 1;
				}
			}
		}
	}
	
	
	// Check for click in tube selector
  if (check == 0)
	  for (i=0+curpos; (i<14+curpos); i++)
	  {
		  CRect tube_box = new CRect(CPoint((i-curpos)*40+60,client.Height()-150), CSize(30, 20));
		  if (tube_box.PtInRect(point))
		  {
			  if (i < m_tube_list->size())
		      text.Format("Tube %s",StripPaddedZeros(m_tube_list->at(i).GetTag()));
          check = 1;
		  }
	  }

  if (check == 1)
  {
    m_tool_tip.AddTool(this, text);
    m_tool_tip.Activate(true);
    SetCursor(AfxGetApp()->LoadStandardCursor(IDC_HAND));
  }
  else
    m_tool_tip.Activate(false);
	  
  CDialog::OnMouseMove(nFlags, point);
}

void ProgressDlg::DrawLegend(CPaintDC *dc)
{
  CRect client;
  GetClientRect(&client);
  DrawWindow(CRect(client.BottomRight() - CSize(150, 90), m_window_size), COLOR_DONE, dc);
  DrawWindow(CRect(client.BottomRight() - CSize(150, 70), m_window_size), COLOR_NOT_ANNOTATED, dc);
  DrawWindow(CRect(client.BottomRight() - CSize(150, 50), m_window_size), COLOR_NOT_EXIST, dc);
}

void ProgressDlg::DrawWindow(const CRect &window, const COLORREF &color, CPaintDC *dc)
{
  CBrush border_brush(RGB(0,0,0));
  CRect border_rect(window.TopLeft(), window.Size());
	CRect box_rect(window.TopLeft(), window.Size() - CSize(1,1));
  //draw a rectangle6
  dc->FrameRect(border_rect, &border_brush);

  //color the rectangle
  dc->FillSolidRect(box_rect, color);
}

void ProgressDlg::ReDraw()
{
  int i, j, k, p;   //loop counters
  int col_length = 22;
  int row_length = 17;

  CPaintDC dc(this); // device context for painting
  int tube_index;
  CRect rect;
  GetClientRect(rect);
  CStatic *Label;
  CString temp;

  if (m_control_size != rect.Size())
  {
    m_control_size = rect.Size();
  }

  DrawLegend(&dc);
  DrawTubeSelect(&dc);

//  if(m_starting_session <= m_jump_size)               //gray out up button
//    m_up_button.EnableWindow(0);
//  if(m_starting_window <= m_jump_size)                //gray out left button
//    m_left_button.EnableWindow(0);
//  if(m_starting_window + m_columns >= m_max_window)  //gray out right button
//    m_right_button.EnableWindow(0);
//  if(m_starting_session + m_rows >= m_max_session)   //gray out down button
//    m_down_button.EnableWindow(0);

  //Determine index of current tube in tube vector
  for(i = 0; i < m_tube_list->size(); i++)
	  if (m_tube_list->at(i).GetTag() == m_displayed_tube)
		  tube_index = i;

  for(i = m_starting_window, k = 1; i < m_columns + m_starting_window - 1; i++, k++)
  {
    if ((k % 5) == 0 || k == 1) {
      temp.Format("%d",i);
      Label = new CStatic;
      Label->Create(temp,WS_CHILD|WS_VISIBLE|SS_SIMPLE,CRect(CPoint((k*col_length), 0) + CSize(25,40), CSize(30,15)),this,0x188);
    }
        
	  for(j = m_starting_session, p = 1; j < m_rows + m_starting_session - 1; j++, p++)
	  {
      if ((p % 5) == 0 || p == 1) {
        temp.Format("%d",j);
        Label = new CStatic;
        Label->Create(temp,WS_CHILD|WS_VISIBLE|SS_RIGHT,CRect(CPoint(0,(p*row_length)) + CSize(15,40), CSize(30,15)),this,0x188);
      }
      //First check if image exists
      if(!(m_tube_list->at(tube_index).GetImageExists(j, i))){
        DrawWindow(CRect(CPoint((k*col_length), (p*row_length)) + CSize(25,40), m_window_size), COLOR_NOT_EXIST, &dc);
      
      //Second Check if image is annotated
      } else if(m_tube_list->at(tube_index).GetImageAnnotated(j, i)){
		    DrawWindow(CRect(CPoint((k*col_length), (p*row_length)) + CSize(25,40), m_window_size), COLOR_DONE, &dc);

      //Else the image is not annotated
      } else {
        DrawWindow(CRect(CPoint((k*col_length), (p*row_length)) + CSize(25,40), m_window_size), COLOR_NOT_ANNOTATED, &dc);
      }
    }
  }
}


//Should there be upper bounds on how far right or down you can go?
void ProgressDlg::OnClickUp() 
{
  if(m_starting_session > m_jump_size)
  {
	  m_starting_session -= m_jump_size;
    m_down_button.EnableWindow(1);
    Invalidate();
  }
}

void ProgressDlg::OnClickRight() 
{
  m_starting_window += m_jump_size;
  m_left_button.EnableWindow(1);
  Invalidate();
}

void ProgressDlg::OnClickLeft() 
{
  if(m_starting_window > m_jump_size)
  {
    m_starting_window -= m_jump_size;
    m_right_button.EnableWindow(1);
    Invalidate();
  }
}

void ProgressDlg::OnClickDown() 
{
  m_starting_session += m_jump_size;
  m_up_button.EnableWindow(1);
  Invalidate();
}

void ProgressDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	int minpos;
  int maxpos;
  pScrollBar->GetScrollRange(&minpos, &maxpos);
  int curpos = pScrollBar->GetScrollPos();

	// Determine the new position of scroll box.
	switch (nSBCode)
	{
	case SB_LEFT:      // Scroll to far left.
		curpos = minpos;
		break;

	case SB_RIGHT:      // Scroll to far right.
		curpos = maxpos;
		break;

	case SB_ENDSCROLL:   // End scroll.
		break;

	case SB_LINELEFT:      // Scroll left.
		if (curpos > minpos)
			curpos--;
		break;

	case SB_LINERIGHT:   // Scroll right.
		if (curpos < maxpos)
			curpos++;
		break;

	case SB_PAGELEFT:    // Scroll one page left.
		{
			// Get the page size. 
			SCROLLINFO   info;
			pScrollBar->GetScrollInfo(&info, SIF_ALL);
   
			if (curpos > minpos)
				curpos = max(minpos, curpos - (int)info.nPage);
		}
		break;

	case SB_PAGERIGHT:      // Scroll one page right
		{
			// Get the page size. 
			SCROLLINFO   info;
			pScrollBar->GetScrollInfo(&info, SIF_ALL);

			if (curpos < maxpos)
				curpos = min(maxpos, curpos + (int) info.nPage);
		}
		break;

	case SB_THUMBPOSITION: // Scroll to absolute position. nPos is the position
		curpos = nPos;      // of the scroll box at the end of the drag operation.
		break;

	case SB_THUMBTRACK:   // Drag scroll box to specified position. nPos is the
		curpos = nPos;     // position that the scroll box has been dragged to.
		break;
	}

	// Set the new position of the thumb (scroll box).
	pScrollBar->SetScrollPos(curpos);
  Invalidate();
	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

void ProgressDlg::DrawTubeSelect(CPaintDC *dc) 
{
  CRect client;
  GetClientRect(&client);
  int curpos = m_sTubeSelect.GetScrollPos();
  int num_tubes = m_tube_list->size();
  int num_exists, num_annotated, displayed_tube;
  COLORREF tube_status;
  CBrush displayed_border(RGB(0,0,255));
  CRect tube_box;
  CStatic *Label;
  int i;

  for(i = 0; i < m_tube_list->size(); i++)
  {
	  if (m_tube_list->at(i).GetTag() == m_displayed_tube)
		  displayed_tube = i;
  }

  for (i=0+curpos; (i<14+curpos); i++)
  {
    tube_box = new CRect(CPoint((i-curpos)*40+60,client.Height()-150), CSize(30, 20));
    if (i<num_tubes)
    {
      Label = new CStatic;
      Label->Create(m_tube_list->at(i).GetTag(),WS_CHILD|WS_VISIBLE|SS_SIMPLE,CRect(tube_box.TopLeft()-CSize(0,20),CSize(30,15)),this,0x188);
      num_exists = m_tube_list->at(i).GetImageNumExists();
      num_annotated = m_tube_list->at(i).GetImageNumAnnotated();
      if (num_exists == num_annotated)
        tube_status = COLOR_DONE;
      else if (num_exists > 0)
        tube_status = COLOR_NOT_ANNOTATED;
      else
        tube_status = COLOR_NOT_EXIST;
    }
    else
      tube_status = COLOR_NOT_EXIST;

    DrawWindow(tube_box, tube_status, dc);

    if (i == displayed_tube)
    {
      dc->FrameRect(tube_box, &displayed_border);
      for (int j=0; j<2; j++)
      {
        tube_box.InflateRect(1,1);
        dc->FrameRect(tube_box, &displayed_border);
      }
    }

  }
}

CString ProgressDlg::StripPaddedZeros(CString input)
{
	int i;
	CString temp;
	for (i = 0; i < input.GetLength(); i++)
	{
		if (input.GetAt(i) != '0') 
		{
			break;
		}
	}
	temp = input.Right(input.GetLength() - i);
	return temp;
}